@@ -1,6 +1,6 @@
 <?php
 /* 
-V4.52 10 Aug 2004  (c) 2000-2004 John Lim (jlim@natsoft.com.my). All rights reserved.
+V4.72 21 Feb 2006  (c) 2000-2006 John Lim (jlim@natsoft.com.my). All rights reserved.
   Released under both BSD license and Lesser GPL library license. 
   Whenever there is any discrepancy between the two licenses, 
   the BSD license will take precedence. 
@@ -19,7 +19,8 @@ Set tabs to 4 for best viewing.
 if (!defined('ADODB_DIR')) die();
 
 if (!defined('_ADODB_ADO_LAYER')) {
-	include(ADODB_DIR."/drivers/adodb-ado.inc.php");
+	if (PHP_VERSION >= 5) include(ADODB_DIR."/drivers/adodb-ado5.inc.php");
+	else include(ADODB_DIR."/drivers/adodb-ado.inc.php");
 }
 
 
@@ -54,35 +55,36 @@ class  ADODB_ado_mssql extends ADODB_ado {
 	
 	function MetaColumns($table)
 	{
-	        $table = strtoupper($table);
-	        $arr= array();
-	        $dbc = $this->_connectionID;
-	        
-	        $osoptions = array();
-	        $osoptions[0] = null;
-	        $osoptions[1] = null;
-	        $osoptions[2] = $table;
-	        $osoptions[3] = null;
-	        
-	        $adors=@$dbc->OpenSchema(4, $osoptions);//tables
+        $table = strtoupper($table);
+        $arr= array();
+        $dbc = $this->_connectionID;
+        
+        $osoptions = array();
+        $osoptions[0] = null;
+        $osoptions[1] = null;
+        $osoptions[2] = $table;
+        $osoptions[3] = null;
+        
+        $adors=@$dbc->OpenSchema(4, $osoptions);//tables
+
+        if ($adors){
+                while (!$adors->EOF){
+                        $fld = new ADOFieldObject();
+                        $c = $adors->Fields(3);
+                        $fld->name = $c->Value;
+                        $fld->type = 'CHAR'; // cannot discover type in ADO!
+                        $fld->max_length = -1;
+                        $arr[strtoupper($fld->name)]=$fld;
+        
+                        $adors->MoveNext();
+                }
+                $adors->Close();
+        }
+        $false = false;
+		return empty($arr) ? $false : $arr;
+	}
 	
-	        if ($adors){
-	                while (!$adors->EOF){
-	                        $fld = new ADOFieldObject();
-	                        $c = $adors->Fields(3);
-	                        $fld->name = $c->Value;
-	                        $fld->type = 'CHAR'; // cannot discover type in ADO!
-	                        $fld->max_length = -1;
-	                        $arr[strtoupper($fld->name)]=$fld;
-	        
-	                        $adors->MoveNext();
-	                }
-	                $adors->Close();
-	        }
-	        
-	        return $arr;
-	}
-	}
+	} // end class 
 	
 	class  ADORecordSet_ado_mssql extends ADORecordSet_ado {        
 	
