@@ -1,6 +1,6 @@
 <?php
 /**
-* @version V4.21 20 Mar 2004 (c) 2000-2004 John Lim (jlim@natsoft.com.my). All rights reserved.
+* @version V4.72 21 Feb 2006 (c) 2000-2006 John Lim (jlim@natsoft.com.my). All rights reserved.
 * Released under both BSD license and Lesser GPL library license.
 * Whenever there is any discrepancy between the two licenses,
 * the BSD license will take precedence.
@@ -12,6 +12,10 @@
 * Informix 9 driver that supports SELECT FIRST
 *
 */
+
+// security - hide paths
+if (!defined('ADODB_DIR')) die();
+
 include_once(ADODB_DIR.'/drivers/adodb-informix72.inc.php');
 
 class ADODB_informix extends ADODB_informix72 {
@@ -22,6 +26,7 @@ class ADODB_informix extends ADODB_informix72 {
 
 class ADORecordset_informix extends ADORecordset_informix72 {
 	var $databaseType = "informix";
+	
 	function ADORecordset_informix($id,$mode=false)
 	{
 		$this->ADORecordset_informix72($id,$mode);
