@@ -1,6 +1,6 @@
 <?php
 /*
-V4.52 10 Aug 2004  (c) 2000-2004 John Lim. All rights reserved.
+V4.72 21 Feb 2006  (c) 2000-2006 John Lim. All rights reserved.
   Released under both BSD license and Lesser GPL library license. 
   Whenever there is any discrepancy between the two licenses, 
   the BSD license will take precedence.
@@ -151,11 +151,19 @@ class ADORecordset_oci8po extends ADORecordset_oci8 {
 	/* Optimize SelectLimit() by using OCIFetch() instead of OCIFetchInto() */
 	function &GetArrayLimit($nrows,$offset=-1) 
 	{
-		if ($offset <= 0) return $this->GetArray($nrows);
+		if ($offset <= 0) {
+			$arr = $this->GetArray($nrows);
+			return $arr;
+		}
 		for ($i=1; $i < $offset; $i++) 
-			if (!@OCIFetch($this->_queryID)) return array();
-			
-		if (!@OCIfetchinto($this->_queryID,$this->fields,$this->fetchMode)) return array();
+			if (!@OCIFetch($this->_queryID)) {
+				$arr = array();
+				return $arr;
+			}
+		if (!@OCIfetchinto($this->_queryID,$this->fields,$this->fetchMode)) {
+			$arr = array();
+			return $arr;
+		}
 		if ($this->fetchMode & OCI_ASSOC) $this->_updatefields();
 		$results = array();
 		$cnt = 0;
