@@ -1,6 +1,6 @@
 <?php
 /* 
-V4.52 10 Aug 2004  (c) 2000-2004 John Lim (jlim@natsoft.com.my). All rights reserved.
+V4.72 21 Feb 2006  (c) 2000-2006 John Lim (jlim@natsoft.com.my). All rights reserved.
   Released under both BSD license and Lesser GPL library license. 
   Whenever there is any discrepancy between the two licenses, 
   the BSD license will take precedence. 
@@ -38,44 +38,42 @@ class  ADODB_odbc_oracle extends ADODB_odbc {
 		
 	function &MetaTables() 
 	{
-		if ($this->metaTablesSQL) {
-			$rs = $this->Execute($this->metaTablesSQL);
-			if ($rs === false) return false;
-			$arr = $rs->GetArray();
-			$arr2 = array();
-			for ($i=0; $i < sizeof($arr); $i++) {
-				$arr2[] = $arr[$i][0];
-			}
-			$rs->Close();
-			return $arr2;
+		$false = false;
+		$rs = $this->Execute($this->metaTablesSQL);
+		if ($rs === false) return $false;
+		$arr = $rs->GetArray();
+		$arr2 = array();
+		for ($i=0; $i < sizeof($arr); $i++) {
+			$arr2[] = $arr[$i][0];
 		}
-		return false;
+		$rs->Close();
+		return $arr2;
 	}
 	
 	function &MetaColumns($table) 
 	{
-		if (!empty($this->metaColumnsSQL)) {
-		
-			$rs = $this->Execute(sprintf($this->metaColumnsSQL,strtoupper($table)));
-			if ($rs === false) return false;
-
-			$retarr = array();
-			while (!$rs->EOF) { //print_r($rs->fields);
-				$fld = new ADOFieldObject();
-				$fld->name = $rs->fields[0];
-				$fld->type = $rs->fields[1];
-				$fld->max_length = $rs->fields[2];
-				
-				
-				if ($ADODB_FETCH_MODE == ADODB_FETCH_NUM) $retarr[] = $fld;	
-				else $retarr[strtoupper($fld->name)] = $fld;
-				
-				$rs->MoveNext();
-			}
-			$rs->Close();
-			return $retarr;	
+	global $ADODB_FETCH_MODE;
+	
+		$rs = $this->Execute(sprintf($this->metaColumnsSQL,strtoupper($table)));
+		if ($rs === false) {	
+			$false = false;
+			return $false;
 		}
-		return false;
+		$retarr = array();
+		while (!$rs->EOF) { //print_r($rs->fields);
+			$fld = new ADOFieldObject();
+			$fld->name = $rs->fields[0];
+			$fld->type = $rs->fields[1];
+			$fld->max_length = $rs->fields[2];
+			
+			
+			if ($ADODB_FETCH_MODE == ADODB_FETCH_NUM) $retarr[] = $fld;	
+			else $retarr[strtoupper($fld->name)] = $fld;
+			
+			$rs->MoveNext();
+		}
+		$rs->Close();
+		return $retarr;	
 	}
 
 	// returns true or false
