@@ -1,6 +1,6 @@
 <?php
 /** 
- * @version V4.21 20 Mar 2004 (c) 2000-2004 John Lim (jlim@natsoft.com.my). All rights reserved.
+ * @version V4.72 21 Feb 2006 (c) 2000-2006 John Lim (jlim@natsoft.com.my). All rights reserved.
  * Released under both BSD license and Lesser GPL library license. 
  * Whenever there is any discrepancy between the two licenses, 
  * the BSD license will take precedence. 
@@ -42,7 +42,14 @@ function &RSFilter($rs,$fn)
 	}
 	$rows = $rs->RecordCount();
 	for ($i=0; $i < $rows; $i++) {
-		$fn($rs->_array[$i],$rs);
+		if (is_array ($fn)) {
+        	$obj = $fn[0];
+        	$method = $fn[1];
+        	$obj->$method ($rs->_array[$i],$rs);
+      } else {
+			$fn($rs->_array[$i],$rs);
+      }
+	  
 	}
 	if (!$rs->EOF) {
 		$rs->_currentRow = 0;
