@@ -1,6 +1,6 @@
 <?php
 /* 
-V4.52 10 Aug 2004  (c) 2000-2004 John Lim (jlim@natsoft.com.my). All rights reserved.
+V4.72 21 Feb 2006  (c) 2000-2006 John Lim (jlim@natsoft.com.my). All rights reserved.
   Released under both BSD license and Lesser GPL library license. 
   Whenever there is any discrepancy between the two licenses, 
   the BSD license will take precedence. 
@@ -18,7 +18,7 @@ include_once('../adodb-pager.inc.php');
 $driver = 'oci8';
 $sql = 'select  ID, firstname as "First Name", lastname as "Last Name" from adoxyz  order  by  id';
 //$sql = 'select count(*),firstname from adoxyz group by firstname order by 2 ';
-$sql = 'select distinct firstname, lastname from adoxyz  order  by  firstname';
+//$sql = 'select distinct firstname, lastname from adoxyz  order  by  firstname';
 
 if ($driver == 'postgres') {
 	$db = NewADOConnection('postgres');
@@ -43,6 +43,9 @@ if ($driver == 'mssql') {
 if ($driver == 'oci8') {
 	$db = NewADOConnection('oci8');
 	$db->Connect('','scott','natsoft');
+	
+$sql = "select * from (select  ID, firstname as \"First Name\", lastname as \"Last Name\" from adoxyz 
+	 order  by  1)";
 }
 
 if ($driver == 'access') {
@@ -52,7 +55,7 @@ if ($driver == 'access') {
 
 if (empty($driver) or $driver == 'mysql') {
 	$db = NewADOConnection('mysql');
-	$db->Connect('localhost','root','','xphplens');
+	$db->Connect('localhost','root','','test');
 }
 
 //$db->pageExecuteCountRows = false;
@@ -65,11 +68,11 @@ include_once('../toexport.inc.php');
 print "<pre>";
 print rs2csv($rs); # return a string
 
-print '<hr>';
+print '<hr />';
 $rs->MoveFirst(); # note, some databases do not support MoveFirst
 print rs2tab($rs); # return a string
 
-print '<hr>';
+print '<hr />';
 $rs->MoveFirst();
 rs2tabout($rs); # send to stdout directly
 print "</pre>";
@@ -77,7 +80,7 @@ print "</pre>";
 
 $pager = new ADODB_Pager($db,$sql);
 $pager->showPageLinks = true;
-$pager->linksPerPage = 3;
+$pager->linksPerPage = 10;
 $pager->cache = 60;
 $pager->Render($rows=7);
 ?>
\ No newline at end of file
