@@ -1,7 +1,7 @@
 <?php
 
 /* 
-V4.52 10 Aug 2004  (c) 2000-2004 John Lim (jlim@natsoft.com.my). All rights reserved.
+V4.72 21 Feb 2006  (c) 2000-2006 John Lim (jlim@natsoft.com.my). All rights reserved.
   Released under both BSD license and Lesser GPL library license. 
   Whenever there is any discrepancy between the two licenses, 
   the BSD license will take precedence. 
@@ -17,9 +17,10 @@ function NotifyExpire($ref,$key)
 
 //-------------------------------------------------------------------
 	
-	
+error_reporting(E_ALL);
+
 #### CONNECTION
-if (1) {
+if (0) {
 	$ADODB_SESSION_DRIVER='oci8';
 	$ADODB_SESSION_CONNECT='';
 	$ADODB_SESSION_USER ='scott';
@@ -34,7 +35,7 @@ if (1) {
 }
 	
 ### TURN DEBUGGING ON
-	$ADODB_SESS_DEBUG = true;
+	$ADODB_SESS_DEBUG = 99;
 
 	
 #### SETUP NOTIFICATION
@@ -44,30 +45,37 @@ if (1) {
 	
 #### INIT
 	ob_start();
-	error_reporting(E_ALL);
 	include('../session/adodb-cryptsession.php');
 	session_start();
 
-
+	adodb_session_regenerate_id();
+	
 ### SETUP SESSION VARIABLES 
-	$HTTP_SESSION_VARS['MONKEY'] = array('1','abc',44.41);
-	if (!isset($HTTP_GET_VARS['nochange'])) @$HTTP_SESSION_VARS['AVAR'] += 1;
+	$_SESSION['MONKEY'] = array('1','abc',44.41);
+	if (!isset($_GET['nochange'])) @$_SESSION['AVAR'] += 1;
 
 	
 ### START DISPLAY
 	print "<h3>PHP ".PHP_VERSION."</h3>";
-	print "<p><b>\$HTTP_SESSION_VARS['AVAR']={$HTTP_SESSION_VARS['AVAR']}</b></p>";
+	print "<p><b>\$_SESSION['AVAR']={$_SESSION['AVAR']}</b></p>";
 	
-	print "<hr> <b>Cookies</b>: ";
-	print_r($HTTP_COOKIE_VARS);
+	print "<hr /> <b>Cookies</b>: ";
+	print_r($_COOKIE);
 	
 ### RANDOMLY PERFORM Garbage Collection
-	if (rand() % 10 == 0) {
+### In real-production environment, this is done for you
+### by php's session extension, which calls adodb_sess_gc()
+### automatically for you. See php.ini's
+### session.cookie_lifetime and session.gc_probability
+
+	if (rand() % 5 == 0) {
 	
-		print "<hr><p><b>Garbage Collection</b></p>";
+		print "<hr /><p><b>Garbage Collection</b></p>";
 		adodb_sess_gc(10);
 		
-		print "<p>Random session destroy</p>";
-		session_destroy();
+		if (rand() % 2 == 0) {
+			print "<p>Random session destroy</p>";
+			session_destroy();
+		}
 	}
 ?>
\ No newline at end of file
